<p align="center" style="text-align:center;color:#381a00;">
<img src="https://totebee.otasys.co.jp/img/site/logo.svg" width=160 style="margin-top:1em;">
  <div align="center">Fetch and Create Contents<br> without Programming.</div>
</p>


## What's This?

ToteBee lets you create data-driven content without programming.
It runs on browsers, fetches CSV/JSON data, and renders them on the page.
<p align="center" style="text-align:center;margin:2em;">
<img src="https://totebee.otasys.co.jp/img/site/illust.svg" width=360 style="margin-top:1em;">
</p>

## Motivation

When we create a web page with a set of data, it is not always necessary to use server side technology or a database. In some cases, it would be nice to complete in 5 minutes without programming.
<br>

## How does it work?

1. Include `totebee.1.0.min.js` into your HTML &lt;HEAD&gt;
2. To specify data source and behaviors, add some `data-` attributes. 
3. Place HTML tags as templates for rendering data.
4. ToteBee automatically fetch the data and render them on the page.
<br>

## Home Page

https://totebee.otasys.co.jp
<p align="center" style="text-align:center;margin-bottom:3em;">
<a href="https://totebee.otasys.co.jp/">
<img src="https://totebee.otasys.co.jp/img/site/capture_home.png" width=200 >
</a>
</p>

## Demos

https://totebee.otasys.co.jp/index.html#demos

## Documents

https://totebee.otasys.co.jp/doc/
<p align="center" style="text-align:center;margin-bottom:3em;">
<a href="https://totebee.otasys.co.jp/doc/">
<img src="https://totebee.otasys.co.jp/img/site/capture_doc.png" width=400>
</a>
</p>

### List

List is the most basic pattern of ToteBee.
It fetches the CSV/JSON data, and render them with the template.

<p align="center" style="text-align:center;margin-bottom:3em;">
<a href="https://totebee.otasys.co.jp/#demo-list"><img src="https://totebee.otasys.co.jp/img/site/demo_list.png?2" style="max-width:600px;" ></a>
</p>




### Table

The list pattern shown above can be apllied to the `<tr>` tag iteration in a `<table>` block.

This pattern can be used to various cases where iteration occurs with the data records.

<p align="center"  style="text-align:center;margin-bottom:3em;">
<a href="https://totebee.otasys.co.jp/#demo-table"><img src="https://totebee.otasys.co.jp/img/site/demo_table.png" style="max-width:600px;" ></a>
</p>

### URL Parameters

Parameters can be passed through the URL without using server side technology.

<p align="center"  style="text-align:center;margin-bottom:3em;">
<a href="https://totebee.otasys.co.jp/#demo-url"><img src="https://totebee.otasys.co.jp/img/site/demo_url.png" style="max-width:600px;" ></a>
</p>


### Record Filter

Record filter is used to extract only those records that fulfill a specified condition.
<p align="center" style="text-align:center;margin-bottom:3em;">
<a href="https://totebee.otasys.co.jp/#demo-filter"><img src="https://totebee.otasys.co.jp/img/site/demo_filter.png" style="max-width:600px;" ></a>
</p>


### If Statement

An If-Statement is used to show a specific HTML block when a record fulfill a specified condition.

<p align="center"  style="text-align:center;margin-bottom:3em;">
<a href="https://totebee.otasys.co.jp/#demo-if"><img src="https://totebee.otasys.co.jp/img/site/demo_if.png" style="max-width:600px;" ></a>
</p>


### Pagenation

Pagination is automatically added by assigning the pagesize.

<p align="center"  style="text-align:center;margin-bottom:3em;">
<a href="https://totebee.otasys.co.jp/#demo-pagenation"><img src="https://totebee.otasys.co.jp/img/site/demo_page.png" style="max-width:600px;" ></a>
</p>

### Sort

<p align="center" style="text-align:center;margin-bottom:3em;">
<a href="https://totebee.otasys.co.jp/#demo-sort"><img src="https://totebee.otasys.co.jp/img/site/demo_sort.png" style="max-width:600px;" ></a>
</p>


### Schedule

You can schedule a content block to go public at a specific date and time.

<p align="center" style="text-align:center;margin-bottom:3em;">
<a href="https://totebee.otasys.co.jp/#demo-schedule"><img src="https://totebee.otasys.co.jp/img/site/demo_sche.png" style="max-width:600px;" ></a>
</p>

## Getting Started

As a sample, we make files like this:

```
.
└── public
    ├── css
    │   └── totebee.css
    ├── data
    │   ├── inventors.csv
    │   └── inventors.json
    ├── js
    │   └── totebee.1.0.min.js
    └── hello.html
```

### Download

1. Download the JS file from [GitHub](https://github.com/naokiota/totebee).
2. Place the JS file `totebee.1.0.min.js` into your project diretory such as `/public/js/`.
3. Add a `<script>` tag  to your `<head>` in `index.html`:
4. If you prefer the same style as the demo page, also add a `<link>` for `totebee.css`.
<br>

```html
<head>
  <title>Awesome Project</title>
  <script type="text/javascript" src="/js/totebee.1.0.min.js"></script>
  <link rel="stylesheet" href="/css/totebee.css" type="text/css">
</head>
```
### Making the data file
As an example, we have a small list of inventors like this: 

|id|name|year|img|
|---|---|---|---|
|101|Thomas Edison|1847-1931|/img/edison.png|
|102|Graham Bell|1847-1922|/img/bell.png|
|103|Nicola Tesla|1856-1943|/img/tesla.png|


ToteBee supports  CSV and JSON (JavaScript Object Notation) format. If you are familiar with JSON format, we recommend using it. JSON files are processed faster than CSV and are often more robust.

  CSV can be converted into JSON online:
   - https://www.csvjson.com/csv2json
   - https://www.freeformatter.com/csv-to-json-converter.html

#### CSV

```csv
id,name,year,img
101,Thomas Edison,1847-1931,/img/edison.png
102,Graham Bell,1847-1922,/img/bell.png
103,Nicola Tesla,1856-1943,/img/tesla.png
```
Make sure that column names(`id,name,year,img`) are defined in the first row. 

If your CSV file includes multibyte characters, choose the `CSV UTF-8(Comma delimited)` to save it.
<img src="https://totebee.otasys.co.jp/img/site/excel.png" width=400>


#### JSON

```Javascript
[{
    "id": "101",
    "name": "Thomas Edison",
    "year": "1847-1931",
    "img": "/img/edison.png"
  },
  {
    "id": "102",
    "name": "Alexander Graham Bell",
    "year": "1847-1922",
    "img": "/img/bell.png"
  },
  {
    "id": "103",
    "name": "Nikola Tesla",
    "year": "1856-1943",
    "img": "/img/tesla.png"
  }
]
```

Please make sure that your JSON file starts with `[` , and ends with `]` . 


**Trailing commas** (sometimes called "final commas") are not supported. 
As you can see below, there is no comma after "value3", "value6", "value9", and the last `}`. 

```Javascript
[
    {
        "key1":"value1",
        "key2":"value2",
        "key3":"value3"  <-- No comma
    },
    {
        "key1":"value4",
        "key2":"value5",
        "key3":"value6"  <-- No comma
    },
    {
        "key1":"value7",
        "key2":"value8",
        "key3":"value9"  <-- No comma
    }  <-- No comma
]
```
Upload the data file to your site before proceeding to the next step. In a browser, enter the URL of the data file and verify that the file can be downloaded from the server.

Verify the data source:

```
https://totebee.otasys.co.jp/data/inventors.csv
```

### Showing a List

<p align="center">
  <img src="https://totebee.otasys.co.jp/img/site/demo_list.png" style="margin:2em;">
</p>

#### HTML

The minimal sample of HTML:

```html
<html>
<head>
  <title>ToteBee</title>
  <link rel="stylesheet" href="/css/totebee.css" type="text/css">
  <script type="text/javascript" src="/js/totebee.1.0.min.js"></script>
</head>
<body>
<h2>ToteBee</h2>
<ul data-x="list" data-format="csv" data-src='/data/inventors.csv'>
    <li data-x="item">
         <img data-src="{img}">
         <div>{name}</div>
         <div>{year}</div>
    </li>
</ul>
</body>
```

##### Specifing the data source

Add a `<ul>` tag with some data attributes:

```html
<ul data-x="list" 
    data-format="csv" 
    data-src="/data/inventors.csv">
```

|attribute|value|meaning|
|---|---|---|
|data-x|list|Binding Shuriken List|
|data-format|csv \| json|Data Format|
|data-src|/data/inventors.csv| Path to your data|

<br>

##### Defining the template

Next, put the `<li>` tag as a template that iterates over the data.
As you can see, each column name is wrapped with curly brackets `{` and `}`.

```html
     <li data-x="item">
         <a href="inventor.html?id={id}">
         <img data-src="{img}">
         <div>{name}</div>
         <div>{year}</div>
       </a>
     </li>
```

|attribute|value|meaning|
|---|---|---|
|data-x|item|template that iterates over the data |
|data-src|{column}|Specify the image path or URL |

When using `<img>` tag, use `data-src` attribute instead of `src`. 

<br>

## More Information

https://totebee.otasys.co.jp


## Contributing

Pull requests are welcome. For major changes, please open an issue first to discuss what you would like to change.
<br>

## License
Copyright (c) 2020 Naoki Ota<br>
Licensed under the [MIT](https://choosealicense.com/licenses/mit/) license.

