/* global fetch */
/* global MutationObserver */
//
// ToteBee
//
(function() {
   "use strict";
    //
    // ToteBeeHelper
    //
    var ToteBeeHelper = {
        //
        // getUrlParameter
        //
        getUrlParameter: function(name) {
            name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
            var regex = new RegExp("[\\?&]" + name + "=([^&#]*)");
            var results = regex.exec(window.location.search);
            return results === null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
        },
        //
        // extract "key" from "url(key)"
        //
        extractUrlKey: function(str) {
            var rx = /url\((.*)\)/g;
            var arr = rx.exec(str);
            return arr[1];
        },
        //
        // detectSign
        //
        detectSign: function(str) {
            var signs = ["<=", ">=", "<", ">", "="];
            for (var i = 0; i < signs.length; i++) {
                var d = signs[i];
                if (str.indexOf(d) > 0) {
                    return d;
                }
            }
            return "";
        },
        //
        // validateFormula
        //
        validateFormula: function(str) {

            var result = {
                error: false,
                sign: "",
                key: "",
                val: ""
            };
            // ex. "id=102" / "id=url(ID)"
            var sign = ToteBeeHelper.detectSign(str);
            if (sign) {

                var ary = str.split(sign);
                var key = ary[0]; // "id"
                var val = ary[1]; // "url(ID)"
                if (key && val) {
                    result.sign = sign;
                    result.key = key;
                    result.val = val;
                    return result;
                }
                else {
                    result.error = true;
                }
            }
            else {
                result.error = true;
            }
            return result;
        },
        //
        // evalValueSet
        // ( left, sign, right ) ex. ("200", ">=", "150") 
        //
        evalValueSet: function(left, sign, right) {
            if (left && sign && right) {
                if (isNaN(left) || isNaN(right)) {
                    return evalValueSetAsString(left, sign, right);
                }
                return evalValueSetAsNumber(left, sign, right);
            }
            return false;

            //
            // evalValueSetAsString
            //
            function evalValueSetAsString(left, sign, right) {
                switch (sign) {
                    case "<":
                        return (left < right);
                    case ">":
                        return (left > right);
                    case "<=":
                        return (left <= right);
                    case ">=":
                        return (left >= right);
                    case "=":
                        return (left == right);
                    default:
                        return false;
                }
            }
            //
            // evalValueSetAsNumber
            //
            function evalValueSetAsNumber(left, sign, right) {
                switch (sign) {
                    case "<":
                        return (parseFloat(left) < parseFloat(right));
                    case ">":
                        return (parseFloat(left) > parseFloat(right));
                    case "<=":
                        return (parseFloat(left) <= parseFloat(right));
                    case ">=":
                        return (parseFloat(left) >= parseFloat(right));
                    case "=":
                        return (parseFloat(left) == parseFloat(right));
                    default:
                        return false;
                }
            }
        },

        //
        // Fetch for IE 11
        //
        ieFetch: function(url, format, callback) {
            var xhr = new XMLHttpRequest();

            xhr.onreadystatechange = function() {
                if (xhr.readyState === 4) {
                    if (xhr.status === 200) {
                        if (format == "json") {
                            callback(JSON.parse(xhr.responseText));
                        }
                        else {
                            callback(xhr.responseText);
                        }
                    }
                }
            };

            xhr.open("GET", url);
            xhr.withCredentials = true;
            xhr.setRequestHeader("Content-Type", "application/json");
            xhr.setRequestHeader("X-Requested-With", "XMLHttpRequest");
            xhr.send();
        },
        //
        //
        //
        isIE: function() {
            return (document.documentMode) ? true : false;
        },
        //
        // pagenate 
        //
        // <ul class="pagenation">
        //	<li class="btn_currpage">1</li>
        //	<li >2</li>
        //	<li >3</li>
        // </ul>
        //
        makePagenateUl: function(datalen, pagesize, currpage, xid) {
            var ul = document.createElement("ul");
            ul.classList.add("pagenation");
            ul.dataset.xid = "pg" + xid;


            var numPage = Math.ceil(datalen / pagesize);
            var pg = 1;
            for (var i = 0; i < numPage; i++) {
                pg = (i + 1);
                
                addPageListItem(ul, pg, currpage, { "xid": xid, "pg": pg });
            }

            return ul;

            //
            // addPageListItem
            //
            function addPageListItem(ul, pg, currpage, dset) {
                var li = document.createElement("li");
                li.appendChild(document.createTextNode(pg));
                li.dataset.xid = dset.xid;
                li.dataset.pg = dset.pg;
                if (pg == currpage) {
                    li.classList.add("btn_currpage");
                }else{
                    li.addEventListener("click", function() {
                        ToteBeeHelper.pagenate(dset.xid, dset.pg);
                    });
                }
                ul.appendChild(li);
            }
        },
        //
        // thinOutPaginate
        //
        thinOutPaginate: function(xid, currpage) {
            // get the pagenation list items
            var XFolderElement = document.querySelector("[data-xid='pg" + xid + "']");
            var aryLi = XFolderElement.querySelectorAll("li");
            if (aryLi.length > 5) {
                var nodeList = Array.prototype.slice.call(aryLi, 0);
                nodeList.forEach(function(li) {

                    // check if the first page
                    if (li.dataset.pg == 1) {
                        li.style.display = "";
                        li.classList.add("btn_firstpage");
                        //li.classList.add("btn_page");
                    }
                    // check if the last page
                    else if (li.dataset.pg == aryLi.length) {
                        li.style.display = "";
                        li.classList.add("btn_lastpage");
                        //li.classList.add("btn_page");
                    }

                    // check if the page is near the current page
                    else if (isNeighborPage(li.dataset.pg, currpage)) {
                        li.style.display = "";
                        if (li.dataset.pg == currpage) {
                            li.classList.add("btn_currpage");
                            //li.classList.remove("btn_page");
                        }
                        else {
                            //li.classList.add("btn_page");
                            li.classList.remove("btn_currpage");
                        }
                    }
                    else {
                            li.style.display = "none";
                    }
                });
            }

            function isNeighborPage(pg, currpage) {
                if (currpage == 1) {
                    return (Math.abs(pg - currpage) < 5);
                }
                else {
                    return (Math.abs(pg - currpage) < 3);
                }
            }
        },

        //
        // pagenate
        //
        pagenate: function(xid, pg) {
            //
            // show and hide items
            //
            var XFolderElement = document.querySelector("[data-xid='" + xid + "']");
            
            var aryLi = XFolderElement.querySelectorAll("li");
            var nodeList = Array.prototype.slice.call(aryLi, 0);
            nodeList.forEach(function(li) {
                if (li.dataset.pg == pg) {
                    li.style.display = "";
                }
                else {
                    li.style.display = "none";
                }
            });	
            
            //
            //
            //
            //XFolderElement.dataset.currpage = pg;

            var currpage = pg;
            var datalen = XFolderElement.dataset.datalen;
            var pagesize = XFolderElement.dataset.pagesize;
            //var currpage = XFolderElement.dataset.currpage;

            // remove existing pagenation
            document.querySelector("[data-xid='pg" + xid + "']").remove();

            // add new pagenation
            var p = ToteBeeHelper.makePagenateUl(datalen, pagesize, currpage, xid);
            ToteBeeHelper.insertAfter(XFolderElement, p);
            ToteBeeHelper.thinOutPaginate(xid, currpage);

        },
        //
        // insertAfter
        //
        insertAfter: function(referenceNode, newNode) {
            referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling);
        },
        //
        // Convert string to date
        //
        str2date: function(str) {
            var d = new Date(str);

            if (d != "Invalid Date") {
                return d;
            }
            else {
                var ary = str.split(" ");
                var sDate = ary[0] || "2000-01-01";
                var sTime = ary[1] || "00:00";
                // yyyy-mm-dd
                var aryDate = sDate.split("-");
                var year = aryDate[0] || "2000";
                var mon = aryDate[1] || "01";
                var day = aryDate[2] || "01";
                // hh-mm-dd
                var aryTime = sTime.split(":");
                var hour = aryTime[0] || "00";
                var min = aryTime[1] || "00";
                var sec = aryTime[2] || "00";

                return new Date(Number(year), Number(mon) - 1, Number(day), Number(hour), Number(min), Number(sec));
            }
        },
        //
        // convert csv to json
        //
        csv2json: function(strData) {
            var jsonObj = [];
            var sDelimiter = ",";
            var arrHeader = [];
            var arrMatches = null;

            // remove blank lines
            strData = strData.replace(/^\s*[\r\n]/gm, "");
            // remove line break at the end of csv file
            strData = strData.replace(/[\r\n]$/, "");
            //
            //
            //
            var regex = new RegExp(
                ("(\\" + sDelimiter + "|\\r?\\n|\\r|^)(?:\"([^\"]*(?:\"\"[^\"]*)*)\"|([^\"\\" + sDelimiter + "\\r\\n]*))"),
                "gi"
            );

            var row = 0,
                col = 0;
            while ((arrMatches = regex.exec(strData))) {

                var delimiter = arrMatches[1];
                if (delimiter.length && delimiter !== sDelimiter) {
                    row++;
                    col = 0;
                    jsonObj.push({});
                }
                var matched;

                if (arrMatches[2]) {
                    // a quoted value.
                    matched = arrMatches[2].replace(new RegExp("\"\"", "g"), "\"");
                }
                else {
                    // a non-quoted value.
                    matched = arrMatches[3];
                }
                if (row == 0) {
                    arrHeader.push(matched);
                }
                else {
                    jsonObj[row - 1][arrHeader[col]] = matched;
                }
                col++;
            }
            return (jsonObj);
        }
    };
    //
    // ToteBee
    //
    var ToteBee = {
        //
        // onReady
        //
        onReady: function(callback) {
            if (document.readyState != "loading") callback();
            else document.addEventListener("DOMContentLoaded", callback);
        },
        //
        // getDatasetAttributesFromElement
        //
        getDatasetAttributesFromElement: function(elm) {

            var xid = elm.dataset.xid || "";
            var currpage = elm.dataset.currpage || "";
            var format = elm.dataset.format || "json"; // [json|csv]
            var filter = elm.dataset.filter || "";
            var src = elm.dataset.src || "";
            var sortkey = elm.dataset.sortkey || "";
            var sortkeyType = elm.dataset.sortkeyType || "string"; // [string|number|date]
            var sortOrder = elm.dataset.sortOrder || "asc"; // [asc|desc]
            var pagesize = elm.dataset.pagesize || "";
            var finish = elm.dataset.finish || "";

            return {
                "xid": xid,
                "currpage": currpage,
                "format": format,
                "filter": filter,
                "src": src,
                "sortkey": sortkey,
                "sortkeyType": sortkeyType,
                "sortOrder": sortOrder,
                "pagesize": pagesize,
                "finish": finish
            };
        },

        //
        // fetchAndRenderByXFolder
        //
        fetchAndRenderByXFolder: function(folder_elm, params) {

            var xid = params.xid || "";
            //var currpage = params.currpage || "";
            var currpage = 1;

            var format = params.format || "json"; // [json|csv]
            var filter = params.filter || "";
            var src = params.src || "";
            var sortkey = params.sortkey || "";
            var sortkeyType = params.sortkeyType || "string"; // [string|number|date]
            var sortOrder = params.sortOrder || "asc"; // [asc|desc]
            var pagesize = params.pagesize || "";
            var finish = params.finish || "";


            var iPage = 1;

            if (!src) {
                return;
            }
            if (format == "csv") {
                fetchCsvData(src);
            }
            else {
                fetchJsonData(src);
            }

            //
            // fetchJsonData
            //
            function fetchJsonData(src) {
                if (ToteBeeHelper.isIE()) {
                    ToteBeeHelper.ieFetch(src, "json", function(res) {
                        pipeLine(res);
                    });
                }
                else {
                    fetch(src)
                        .then(function(res) {
                            return res.json();
                        })
                        .then(function(jsonData) {
                            //
                            // hand over data to pipeLine
                            //
                            pipeLine(jsonData);
                        });
                }
            }
            //
            // fetchCsvData
            //
            function fetchCsvData(src) {
                if (ToteBeeHelper.isIE()) {

                    ToteBeeHelper.ieFetch(src, "csv", function(res) {
                        var jsonData = ToteBeeHelper.csv2json(res);
                        pipeLine(jsonData);
                    });
                }
                else {
                    fetch(src)
                        .then(function(response) {
                            return response.text();
                        })
                        .then(function(text) {
                            // CSV to JSON
                            var jsonData = ToteBeeHelper.csv2json(text);
                            //
                            // hand over data to pipeLine
                            //
                            pipeLine(jsonData);
                        });
                }
            }

            //
            // pipeLine
            //
            function pipeLine(jsonData) {
                //
                // pipe filter
                // 
                if (filter) {
                    jsonData = pipeFilterJson(jsonData, filter);
                }
                //
                // pipe sort
                // 
                if (sortkey) {
                    jsonData = pipeSortJson(jsonData, sortkey);
                }
                //
                // pipe render
                //
                pipeRenderJson(jsonData);
                //
                // finishing call
                //
                if (finish) {
                    finishingCall(finish);
                }
            }
            //
            // pipeFilterJson
            // 
            // filter example : "id=101"
            // filter example : "id=url(ID)"
            // URL example : detail.html?ID=101
            //
            //
            function pipeFilterJson(jsonData, strFilter) {

                // ex. "id=102" / "id<=url(ID)"
                var res = ToteBeeHelper.validateFormula(strFilter);
                if (res.error) {
                    return jsonData;
                }
                //console.log("sign", res.sign); // "=""
                //console.log("key", res.key); // "id"
                //console.log("val", res.val); // "102"

                var jsonObj = [];

                //
                // retrieve the value from the url parameter.
                //
                if (res.val.indexOf("url") >= 0) {
                    var urlKey = ToteBeeHelper.extractUrlKey(res.val); // "ID"
                    var urlVal = ToteBeeHelper.getUrlParameter(urlKey); // 101
                    res.val = urlVal;
                }
                //
                // filtering
                //
                for (var i = 0; i < jsonData.length; i++) {
                    //
                    // evaluate value set
                    //
                    if (ToteBeeHelper.evalValueSet(jsonData[i][res.key], res.sign, res.val)) {
                        //
                        // push data into return-object
                        //
                        jsonObj.push(jsonData[i]);
                    }
                }
                return jsonObj;



            }

            //
            // pipeSortJson
            //
            function pipeSortJson(jsonData) {

                //
                // Sort Json Array
                //
                if (sortkey) {
                    jsonData = jsonData.sort(sortJsonArray(sortkey, sortkeyType, sortOrder));
                }
                //
                // Return Sorted Object
                //
                return jsonData;
                //
                // sortJsonArray
                //
                function sortJsonArray(key, keytype) {
                    //
                    // Compare as number
                    //
                    if (keytype == "number") {
                        // descending
                        if (sortOrder == "desc") {
                            return function(a, b) {
                                if (parseFloat(a[key]) < parseFloat(b[key])) { return 1; }
                                else if (parseFloat(a[key]) > parseFloat(b[key])) { return -1; }
                                return 0;
                            };
                        }
                        // ascending
                        else {
                            return function(a, b) {
                                if (parseFloat(a[key]) > parseFloat(b[key])) { return 1; }
                                else if (parseFloat(a[key]) < parseFloat(b[key])) { return -1; }
                                return 0;
                            };
                        }
                    }
                    //
                    // Compare as date
                    //
                    else if (keytype == "date") {
                        // descending
                        if (sortOrder == "desc") {
                            return function(a, b) {
                                if (new Date(a[key]) < new Date(b[key])) { return 1; }
                                else if (new Date(a[key]) > new Date(b[key])) { return -1; }
                                return 0;
                            };
                        }
                        // ascending
                        else {
                            return function(a, b) {
                                if (new Date(a[key]) > new Date(b[key])) { return 1; }
                                else if (new Date(a[key]) < new Date(b[key])) { return -1; }
                                return 0;
                            };
                        }
                    }
                    //
                    // Compare as string
                    //
                    else {
                        // descending
                        if (sortOrder == "desc") {
                            return function(a, b) {
                                if (a[key] < b[key]) { return 1; }
                                else if (a[key] > b[key]) { return -1; }
                                return 0;
                            };
                        }
                        // ascending
                        else {
                            return function(a, b) {
                                if (a[key] > b[key]) { return 1; }
                                else if (a[key] < b[key]) { return -1; }
                                return 0;
                            };
                        }
                    }
                }
            }

            //
            // pipe RenderJson
            //
            function pipeRenderJson(jsonData) {

                var items = folder_elm.querySelectorAll("[data-x='item']");

                if (items.length < 1) {
                    console.log("Warn : No [data-x='item'] was found.");
                    return;
                }
                var item = items[0]; // template
                jsonData.forEach(function(jsonRecord, idx) {

                    var visible = true;
                    if (pagesize) {
                        var iPMax = iPage * pagesize;
                        if (idx >= iPMax) {
                            visible = false;
                        }
                    }

                    var pd = item.cloneNode(true);
                    var template = pd.innerHTML;
                    for (var key in jsonRecord) {
                        if (jsonRecord.hasOwnProperty(key)) {
                            var val = jsonRecord[key];
                            template = template.replace(new RegExp("\\{" + key + "\\}", "g"), val);

                        }
                    }
                    pd.innerHTML = template;

                    //
                    // evaluate If
                    //
                    for (var ky in jsonRecord) {
                        if (jsonRecord.hasOwnProperty(ky)) {
                            var value = jsonRecord[ky];
                            //
                            // evaluate If
                            //
                            evalIf(pd, ky, value);
                        }
                    }
                    //
                    // make texts short
                    //
                    maxTextLen(pd);
                    //
                    // img tag
                    //
                    var imgs = pd.getElementsByTagName("img");
                    for (var i = 0; i < imgs.length; i++) {
                        var src = imgs[i].getAttribute("data-src");
                        if(src){
                            pd.getElementsByTagName("img")[i].setAttribute("src", src);
                        }
                    }

                    if (visible) {
                        //pd.style.visibility = "visible";
                    }
                    else {
                        pd.style.display = "none";
                    }
                    pd.dataset.pg = Math.ceil((idx + 1) / pagesize);
                    if (ToteBeeHelper.isIE()) {
                        folder_elm.appendChild(pd);
                    }
                    else {
                        folder_elm.append(pd);
                    }
                });

                item.style.display = "none";
                if (pagesize) {
                    folder_elm.dataset.datalen = jsonData.length;
                    if (jsonData.length > pagesize) {

                        var p = ToteBeeHelper.makePagenateUl(jsonData.length, pagesize, currpage, xid);
                        ToteBeeHelper.insertAfter(folder_elm, p);
                        ToteBeeHelper.thinOutPaginate(xid, currpage);
                    }
                }

                //
                // Evaluate If Statement
                //
                // <div data-x="if" data-condition="id=102">
                // 
                function evalIf(pd, key, val) {

                    var divs = pd.querySelectorAll("[data-x='if']");
                    var nodeList = Array.prototype.slice.call(divs, 0);
                    nodeList.forEach(function(div) {

                        // ex. "id=102" / "id<=url(ID)"
                        var condition = div.dataset.condition;
                        if (condition) {

                            // ex. "id=102"
                            var res = ToteBeeHelper.validateFormula(condition);
                            if (res.error) {
                                return;
                            }
                            // res.key  : "id"
                            // res.sign : "="
                            // res.val  : "102"
                            if (key == res.key) {
                                if (ToteBeeHelper.evalValueSet(val, res.sign, res.val)) {
                                    div.style.display = "block";
                                }
                                else {
                                    div.style.display = "none";
                                }
                            }
                        }
                    });
                }
                //
                // maxTextLen
                //
                function maxTextLen(pd) {
                    var divs = pd.querySelectorAll("[data-x='maxlen']");
                    var nodeList = Array.prototype.slice.call(divs, 0);
                    nodeList.forEach(function(div) {
                        var len = div.dataset.len || 0;
                        var replace = div.dataset.replace || "...";
                        var txt = div.innerText || "";

                        if ((len > 0) && (txt.length > len)) {
                            div.innerText = txt.substring(0, len) + replace;
                        }
                    });
                }
            }
            //
            // finishingCall
            //
            function finishingCall(fname) {
                if (fname) {
                    // find object
                    var fn = window[fname];

                    // is object a function?
                    if (typeof fn === "function") {
                        fn();
                    }
                }
            }
        },

        //
        // listProcess
        //
        listProcess: function() {

            //
            // querySelectorAll
            //
            var XFolderElements = document.querySelectorAll('[data-x="list"]');
            var nodeList = Array.prototype.slice.call(XFolderElements, 0);
            var i = 0;
            nodeList.forEach(function(folder_elm) {

                folder_elm.dataset.xid = i++;
                folder_elm.dataset.currpage = 1;
                if (!folder_elm.dataset.flg) {
                    folder_elm.dataset.flg = true; //
                    // Get Data Attributes
                    //
                    var params = ToteBee.getDatasetAttributesFromElement(folder_elm);
                    //
                    // fetch and Render
                    //
                    ToteBee.fetchAndRenderByXFolder(folder_elm, params);
                }
            });
        },
        //
        // reloadProcess
        //
        reloadProcess: function() {
            //
            // Set Click Event Handlers for Reload/Sort
            //
            var ButtonElements = document.querySelectorAll("[data-x='reload']");
            var nodeList = Array.prototype.slice.call(ButtonElements, 0);
            nodeList.forEach(function(btn_elm) {
                //
                // Set onClick Event Hander
                //
                btn_elm.onclick = function(e) {

                    //
                    // Get Data Attributes
                    //
                    var tgt_elm = document.getElementById(e.target.dataset.targetid);
                    var xid = e.target.dataset.xid || tgt_elm.dataset.xid;
                    var pagesize = e.target.dataset.pagesize || tgt_elm.dataset.pagesize;
                    var format = e.target.dataset.format || tgt_elm.dataset.format;
                    var filter = e.target.dataset.filter || tgt_elm.dataset.filter;
                    var src = e.target.dataset.src || tgt_elm.dataset.src;
                    var sortkey = e.target.dataset.sortkey || tgt_elm.dataset.sortkey;
                    var sortkeyType = e.target.dataset.sortkeyType || tgt_elm.dataset.sortkeyType; // string / number / date
                    var sortOrder = e.target.dataset.sortOrder || tgt_elm.dataset.sortOrder; // asc / desc
                    var finish = e.target.dataset.finish || tgt_elm.dataset.finish;

                    //
                    // Set Parameter Object
                    //
                    var params = {
                        "xid": xid,
                        "format": format,
                        "filter": filter,
                        "src": src,
                        "sortkey": sortkey,
                        "sortkeyType": sortkeyType,
                        "sortOrder": sortOrder,
                        "pagesize": pagesize,
                        "finish": finish
                    };
                    //
                    // Clear Existing Items in XFolder
                    //
                    clearExistingItemsInXFolder(tgt_elm);
                    //
                    // Remove Active-Attribute for Same Target
                    //
                    removeActiveForSameTarget(e.target.dataset.targetid);
                    //
                    // Add 
                    //
                    e.target.classList.add("active");

                    // remove existing pagenation
                    document.querySelector("[data-xid='pg" + xid + "']").remove();

                    //
                    // fetchAndRenderByXFolder
                    //
                    ToteBee.fetchAndRenderByXFolder(tgt_elm, params);

                };
            });
            //
            // Clear Existing Items In XFolder
            //
            function clearExistingItemsInXFolder(folder_elm) {
                var items = folder_elm.querySelectorAll("[data-x='item']");
                var nodeList = Array.prototype.slice.call(items, 0);
                nodeList.forEach(function(item, i) {
                    if (i == 0) {
                        item.style.display = "";
                    }
                    else {
                        item.remove();
                    }
                });
            }
            //
            // Remove Active For Same Target
            //
            function removeActiveForSameTarget(targetid) {
                var buttons = document.querySelectorAll("[data-targetid='" + targetid + "']");
                var nodeList = Array.prototype.slice.call(buttons, 0);
                nodeList.forEach(function(btn) {
                    btn.classList.remove("active");
                });
            }
        },
        scheduleProcess: function() {
            //
            // querySelectorAll
            //
            var elements = document.querySelectorAll("[data-x='schedule']");
            var nodeList = Array.prototype.slice.call(elements, 0);
            //
            //
            //
            nodeList.forEach(function(elm) {

                var dt_start = elm.dataset.start || "";
                var dt_end = elm.dataset.end || "";
                //
                // check start date if specified
                //
                var start_show = true;
                if (dt_start) {
                    start_show = (ToteBeeHelper.str2date(dt_start) < Date.now());
                }
                //
                // check end date if specified
                //
                var keep_show = true;
                if (dt_end) {
                    keep_show = (Date.now() < ToteBeeHelper.str2date(dt_end));
                }
                //
                // show or hide the block
                //
                if (start_show && keep_show) {
                    elm.style.display = "block";
                }
                else {
                    elm.style.display = "none";
                }
            });
        },
        ///
        // run ToteBee
        //
        run: function() {

            var body = document.getElementsByTagName("body")[0];
            var mo = new MutationObserver(function(mutations){
                mutations.forEach(function(mutation){
                    ToteBee.listProcess(mutation.target);
                });
            });

            var moConfig = {
                childList: true,
                subtree: true
            };
            mo.observe(body, moConfig);
            this.listProcess(body);
            this.reloadProcess();
            this.scheduleProcess();
        }
    };
    ToteBee.onReady(function() {
        ToteBee.run();
    });
})();
